      subroutine rotrecur (R1, Rold, Rnew, l)
c
c.....rotrecur - obtain Wigner rotation matrix R^l from R^(l-1) and R^1
c
c     Routine based on the recursive algorithm by Ivanic and Ruedenberg,
c     J. Phys. Chem. 100, 6342-47 (1996).
c
c.....IMPORTANT NOTE: Ivanic and Ruedenberg defined the coordinate
c     transformation matrix acting on row vectors, and obtained the
c     rotation matrices acting on row vectors, but considering that
c     they should give the RSH value in the rotated point Slm(\hat{R}x);
c     since we want the rotation matrices (still acting on row vectors)
c     for the rotation of the functions,
c       \hat{R} Slm(x) = Slm(R^{-1}x),
c     which is the transpose of the above one, and we work with
c     coordinate transformation matrices acting on column vectors, again
c     the transpose of Ivanic and Ruedenberg definition, we can just use
c     the same algorithm and the double transpose will give our desired
c     convention as output; thus we will obtain:
c       \hat{R} S_lm2(x) = S_lm2(R^{-1}x) = \sum_m1 S_lm1(x) R^l_m1m2
c     We will assume that l>1, loops like do i = 2, 1 should not
c     execute, and do i = 2, 2 should run one trip in order for the l=2
c     and l=3 cases to give the correct answer (this is the standard
c     Fortran 77 behavior).
c
      include           'implicit.inc'
      include           'consts.inc'
c
c.....Parameters:
c
c     R1(,) ........... rotation matrix for L=1
c     Rold(,) ......... rotation matrix for L=l-1
c     Rnew(,) ......... rotation matrix for L=l
c     l ............... angular momentum L of the required matrix
c
      real*8            R1(-1:1,-1:1), Rold(1-l:l-1,1-l:l-1)
      real*8            Rnew(-l:l,-l:l)
c
      real*8            u, v, w, uu, vv, ww, den
      integer           iu, iv, iw
c
c.....loop over all values of |m1| and |m2|
c
c.....m1 = 0
c       m2 = 0
c
      v = - sqrt((l-1)/(two*l))
      uu = R1(0,0) * Rold(0,0)
      vv = R1(1,0) * Rold(1,0) + R1(-1,0) * Rold(-1,0)
*     u = 1,   ww = 0
      Rnew(0,0) = uu + v * vv
c
c.....m1 = 0
c       0 < m2 < l
c
      iu = l*l
      iv = 2*l*(l-1)
      do m2 = 1, l-1
        den = (l-m2) * (l+m2)
        u = sqrt(iu/den)
        v = -half * sqrt(iv/den)
        uu = R1(0,0) * Rold(0,m2)
        vv = R1(1,0) * Rold(1,m2) + R1(-1,0) * Rold(-1,m2)
*       ww = 0
        Rnew(0,m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(0,-m2)
        vv = R1(1,0) * Rold(1,-m2) + R1(-1,0) * Rold(-1,-m2)
        Rnew(0,-m2) = u * uu + v * vv
      enddo
c
c.....m1 = 0
c       m2 = l
c
      u = sqrt(l/(two*(l+l-1)))
      v = -half * sqrt((l-1)/(l+l-one))
      uu = R1(0,1) * Rold(0,l-1) - R1(0,-1) * Rold(0,1-l)
      vv = R1(1,1) * Rold(1,l-1) - R1(1,-1) * Rold(1,1-l) +
     &     R1(-1,1) * Rold(-1,l-1) - R1(-1,-1) * Rold(-1,1-l)
*     ww = 0
      Rnew(0,l) = u * uu + v * vv
      uu = R1(0,1) * Rold(0,1-l) + R1(0,-1) * Rold(0,l-1)
      vv = R1(1,1) * Rold(1,1-l) + R1(1,-1) * Rold(1,l-1) +
     &     R1(-1,1) * Rold(-1,1-l) + R1(-1,-1) * Rold(-1,l-1)
      Rnew(0,-l) = u * uu + v * vv
c
c.....m1 = 1
c       m2 = 0
c
      iu = (l-1) * (l+1)
*     sqrt(2)*v, coming from vv
      iv = 2 * l * (l+1)
      iw = (l-2) * (l-1)
*
      den = l * l
      u = sqrt(iu/den)
      v = half * sqrt(iv/den)
      uu = R1(0,0) * Rold(1,0)
      vv = R1(1,0) * Rold(0,0)
      Rnew(1,0) = u * uu + v * vv
      uu = R1(0,0) * Rold(-1,0)
      vv = R1(-1,0) * Rold(0,0)
      Rnew(-1,0) = u * uu + v * vv
      if (iw.ne.0) then
        w = -half * sqrt(iw/den)
        ww = R1(1,0) * Rold(2,0) + R1(-1,0) * Rold(-2,0)
        Rnew(1,0) = Rnew(1,0) + w * ww
        ww = R1(1,0) * Rold(-2,0) - R1(-1,0) * Rold(2,0)
        Rnew(-1,0) = Rnew(-1,0) + w * ww
      endif
c
c.....m1 = 1
c       0 < m2 < l
c
      do m2 = 1, l-1
        den = (l-m2) * (l+m2)
        u = sqrt(iu/den)
        v = half * sqrt(iv/den)
        uu = R1(0,0) * Rold(1,m2)
        vv = R1(1,0) * Rold(0,m2)
        Rnew(1,m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(1,-m2)
        vv = R1(1,0) * Rold(0,-m2)
        Rnew(1,-m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(-1,m2)
        vv = R1(-1,0) * Rold(0,m2)
        Rnew(-1,m2) = u * uu + v * vv
        uu = R1(0,0) * Rold(-1,-m2)
        vv = R1(-1,0) * Rold(0,-m2)
        Rnew(-1,-m2) = u * uu + v * vv
        if (iw.ne.0) then
          w = -half * sqrt(iw/den)
          ww = R1(1,0) * Rold(2,m2) + R1(-1,0) * Rold(-2,m2)
          Rnew(1,m2) = Rnew(1,m2) + w * ww
          ww = R1(1,0) * Rold(2,-m2) + R1(-1,0) * Rold(-2,-m2)
          Rnew(1,-m2) = Rnew(1,-m2) + w * ww
          ww = R1(1,0) * Rold(-2,m2) - R1(-1,0) * Rold(2,m2)
          Rnew(-1,m2) = Rnew(-1,m2) + w * ww
          ww = R1(1,0) * Rold(-2,-m2) - R1(-1,0) * Rold(2,-m2)
          Rnew(-1,-m2) = Rnew(-1,-m2) + w * ww
        endif
      enddo
c
c.....m1 = 1
c       m2 = l
c
      den = 2 * l * (l+l-1)
      u = sqrt(iu/den)
      v = half * sqrt(iv/den)
      uu = R1(0,1) * Rold(1,l-1) - R1(0,-1) * Rold(1,1-l)
      vv = R1(1,1) * Rold(0,l-1) - R1(1,-1) * Rold(0,1-l)
      Rnew(1,l) = u * uu + v * vv
      uu = R1(0,1) * Rold(1,1-l) + R1(0,-1) * Rold(1,l-1)
      vv = R1(1,1) * Rold(0,1-l) + R1(1,-1) * Rold(0,l-1)
      Rnew(1,-l) = u * uu + v * vv
      uu = R1(0,1) * Rold(-1,l-1) - R1(0,-1) * Rold(-1,1-l)
      vv = R1(-1,1) * Rold(0,l-1) - R1(-1,-1) * Rold(0,1-l)
      Rnew(-1,l) = u * uu + v * vv
      uu = R1(0,1) * Rold(-1,1-l) + R1(0,-1) * Rold(-1,l-1)
      vv = R1(-1,1) * Rold(0,1-l) + R1(-1,-1) * Rold(0,l-1)
      Rnew(-1,-l) = u * uu + v * vv
      if (iw.ne.0) then
        w = -half * sqrt(iw/den)
        ww = R1(1,1) * Rold(2,l-1) - R1(1,-1) * Rold(2,1-l) +
     &       R1(-1,1) * Rold(-2,l-1) - R1(-1,-1) * Rold(-2,1-l)
        Rnew(1,l) = Rnew(1,l) + w * ww
        ww = R1(1,1) * Rold(2,1-l) + R1(1,-1) * Rold(2,l-1) +
     &       R1(-1,1) * Rold(-2,1-l) + R1(-1,-1) * Rold(-2,l-1)
        Rnew(1,-l) = Rnew(1,-l) + w * ww
        ww = R1(1,1) * Rold(-2,l-1) - R1(1,-1) * Rold(-2,1-l) -
     &       R1(-1,1) * Rold(2,l-1) + R1(-1,-1) * Rold(2,1-l)
        Rnew(-1,l) = Rnew(-1,l) + w * ww
        ww = R1(1,1) * Rold(-2,1-l) + R1(1,-1) * Rold(-2,l-1) -
     &       R1(-1,1) * Rold(2,1-l) - R1(-1,-1) * Rold(2,l-1)
        Rnew(-1,-l) = Rnew(-1,-l) + w * ww
      endif
c
c.....1 < m1 < l
c       m2 = 0
c
      do m1 = 2, l-1
        iu = (l-m1) * (l+m1)
        iv = (l-1+m1) * (l+m1)
        iw = (l-1-m1) * (l-m1)
*
        den = l * l
        u = sqrt(iu/den)
        v = half * sqrt(iv/den)
        uu = R1(0,0) * Rold(m1,0)
        vv = R1(1,0) * Rold(m1-1,0) - R1(-1,0) * Rold(1-m1,0)
        Rnew(m1,0) = u * uu + v * vv
        uu = R1(0,0) * Rold(-m1,0)
        vv = R1(1,0) * Rold(1-m1,0) + R1(-1,0) * Rold(m1-1,0)
        Rnew(-m1,0) = u * uu + v * vv
        if (iw.ne.0) then
          w = -half * sqrt(iw/den)
          ww = R1(1,0) * Rold(m1+1,0) + R1(-1,0) * Rold(-m1-1,0)
          Rnew(m1,0) = Rnew(m1,0) + w * ww
          ww = R1(1,0) * Rold(-m1-1,0) - R1(-1,0) * Rold(m1+1,0)
          Rnew(-m1,0) = Rnew(-m1,0) + w * ww
        endif
c
c.....1 < m1 < l
c       0 < m2 < l
c
        do m2 = 1, l-1
          den = (l-m2) * (l+m2)
          u = sqrt(iu/den)
          v = half * sqrt(iv/den)
          uu = R1(0,0) * Rold(m1,m2)
          vv = R1(1,0) * Rold(m1-1,m2) - R1(-1,0) * Rold(1-m1,m2)
          Rnew(m1,m2) = u * uu + v * vv
          uu = R1(0,0) * Rold(m1,-m2)
          vv = R1(1,0) * Rold(m1-1,-m2) - R1(-1,0) * Rold(1-m1,-m2)
          Rnew(m1,-m2) = u * uu + v * vv
          uu = R1(0,0) * Rold(-m1,m2)
          vv = R1(1,0) * Rold(1-m1,m2) + R1(-1,0) * Rold(m1-1,m2)
          Rnew(-m1,m2) = u * uu + v * vv
          uu = R1(0,0) * Rold(-m1,-m2)
          vv = R1(1,0) * Rold(1-m1,-m2) + R1(-1,0) * Rold(m1-1,-m2)
          Rnew(-m1,-m2) = u * uu + v * vv
          if (iw.ne.0) then
            w = -half * sqrt(iw/den)
            ww = R1(1,0) * Rold(m1+1,m2) + R1(-1,0) * Rold(-m1-1,m2)
            Rnew(m1,m2) = Rnew(m1,m2) + w * ww
            ww = R1(1,0) * Rold(m1+1,-m2) + R1(-1,0) * Rold(-m1-1,-m2)
            Rnew(m1,-m2) = Rnew(m1,-m2) + w * ww
            ww = R1(1,0) * Rold(-m1-1,m2) - R1(-1,0) * Rold(m1+1,m2)
            Rnew(-m1,m2) = Rnew(-m1,m2) + w * ww
            ww = R1(1,0) * Rold(-m1-1,-m2) - R1(-1,0) * Rold(m1+1,-m2)
            Rnew(-m1,-m2) = Rnew(-m1,-m2) + w * ww
          endif
        enddo
c
c.....1 < m1 < l
c       m2 = l
c
        den = 2 * l * (l+l-1)
        u = sqrt(iu/den)
        v = half * sqrt(iv/den)
        uu = R1(0,1) * Rold(m1,l-1) - R1(0,-1) * Rold(m1,1-l)
        vv = R1(1,1) * Rold(m1-1,l-1) - R1(1,-1) * Rold(m1-1,1-l) -
     &       R1(-1,1) * Rold(1-m1,l-1) + R1(-1,-1) * Rold(1-m1,1-l)
        Rnew(m1,l) = u * uu + v * vv
        uu = R1(0,1) * Rold(m1,1-l) + R1(0,-1) * Rold(m1,l-1)
        vv = R1(1,1) * Rold(m1-1,1-l) + R1(1,-1) * Rold(m1-1,l-1) -
     &       R1(-1,1) * Rold(1-m1,1-l) - R1(-1,-1) * Rold(1-m1,l-1)
        Rnew(m1,-l) = u * uu + v * vv
        uu = R1(0,1) * Rold(-m1,l-1) - R1(0,-1) * Rold(-m1,1-l)
        vv = R1(1,1) * Rold(1-m1,l-1) - R1(1,-1) * Rold(1-m1,1-l) +
     &       R1(-1,1) * Rold(m1-1,l-1) - R1(-1,-1) * Rold(m1-1,1-l)
        Rnew(-m1,l) = u * uu + v * vv
        uu = R1(0,1) * Rold(-m1,1-l) + R1(0,-1) * Rold(-m1,l-1)
        vv = R1(1,1) * Rold(1-m1,1-l) + R1(1,-1) * Rold(1-m1,l-1) +
     &       R1(-1,1) * Rold(m1-1,1-l) + R1(-1,-1) * Rold(m1-1,l-1)
        Rnew(-m1,-l) = u * uu + v * vv
        if (iw.ne.0) then
          w = -half * sqrt(iw/den)
          ww = R1(1,1) * Rold(m1+1,l-1) - R1(1,-1) * Rold(m1+1,1-l) +
     &         R1(-1,1) * Rold(-1-m1,l-1) - R1(-1,-1) * Rold(-1-m1,1-l)
          Rnew(m1,l) = Rnew(m1,l) + w * ww
          ww = R1(1,1) * Rold(m1+1,1-l) + R1(1,-1) * Rold(m1+1,l-1) +
     &         R1(-1,1) * Rold(-1-m1,1-l) + R1(-1,-1) * Rold(-1-m1,l-1)
          Rnew(m1,-l) = Rnew(m1,-l) + w * ww
          ww = R1(1,1) * Rold(-1-m1,l-1) - R1(1,-1) * Rold(-1-m1,1-l) -
     &         R1(-1,1) * Rold(m1+1,l-1) + R1(-1,-1) * Rold(m1+1,1-l)
          Rnew(-m1,l) = Rnew(-m1,l) + w * ww
          ww = R1(1,1) * Rold(-1-m1,1-l) + R1(1,-1) * Rold(-1-m1,l-1) -
     &         R1(-1,1) * Rold(m1+1,1-l) - R1(-1,-1) * Rold(m1+1,l-1)
          Rnew(-m1,-l) = Rnew(-m1,-l) + w * ww
        endif
      enddo
c
c.....m1 = l
c       m2 = 0
c
*     u, w = 0
      v = sqrt((l+l-one)/(l+l))
      vv = R1(1,0) * Rold(l-1,0) - R1(-1,0) * Rold(1-l,0)
      Rnew(l,0) = v * vv
      vv = R1(1,0) * Rold(1-l,0) + R1(-1,0) * Rold(l-1,0)
      Rnew(-l,0) = v * vv
c
c.....m1 = l
c       0 < m2 < l
c
      iv = (l+l) * (l+l-1)
      do m2 = 1, l-1
        den = (l-m2) * (l+m2)
        v = half * sqrt(iv/den)
        vv = R1(1,0) * Rold(l-1,m2) - R1(-1,0) * Rold(1-l,m2)
        Rnew(l,m2) = v * vv
        vv = R1(1,0) * Rold(l-1,-m2) - R1(-1,0) * Rold(1-l,-m2)
        Rnew(l,-m2) = v * vv
        vv = R1(1,0) * Rold(1-l,m2) + R1(-1,0) * Rold(l-1,m2)
        Rnew(-l,m2) = v * vv
        vv = R1(1,0) * Rold(1-l,-m2) + R1(-1,0) * Rold(l-1,-m2)
        Rnew(-l,-m2) = v * vv
      enddo
c
c.....m1 = l
c       m2 = l
c
      v = half
      vv = R1(1,1) * Rold(l-1,l-1) - R1(1,-1) * Rold(l-1,1-l) -
     &     R1(-1,1) * Rold(1-l,l-1) + R1(-1,-1) * Rold(1-l,1-l)
      Rnew(l,l) = v * vv
      vv = R1(1,1) * Rold(l-1,1-l) + R1(1,-1) * Rold(l-1,l-1) -
     &     R1(-1,1) * Rold(1-l,1-l) - R1(-1,-1) * Rold(1-l,l-1)
      Rnew(l,-l) = v * vv
      vv = R1(1,1) * Rold(1-l,l-1) - R1(1,-1) * Rold(1-l,1-l) +
     &     R1(-1,1) * Rold(l-1,l-1) - R1(-1,-1) * Rold(l-1,1-l)
      Rnew(-l,l) = v * vv
      vv = R1(1,1) * Rold(1-l,1-l) + R1(1,-1) * Rold(1-l,l-1) +
     &     R1(-1,1) * Rold(l-1,1-l) + R1(-1,-1) * Rold(l-1,l-1)
      Rnew(-l,-l) = v * vv
c
c.....routine ends
c
      return
      end
